//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "stdafx.h"
#include "types.h"
#include "listensocket.h"
#include "partfile.h"
#include "clientcredits.h"
#include "safefile.h"
#include "BarShader.h"
#include "StoredSources.h" // Added by Tarod (LS tweak)
#include "Loggable.h"
#include <vector>


// m_byChatstate
#define	MS_NONE				0
#define	MS_CHATTING			1
#define	MS_CONNECTING		2
#define	MS_UNABLETOCONNECT	3



class CClientReqSocket;
class CFriend;

class CUpDownClient : public CLoggable
{
	friend class CUploadQueue;
public:
	//base
	CUpDownClient(CClientReqSocket* sender = 0);
	CUpDownClient(uint16 in_port, uint32 in_userid, uint32 in_serverup, uint16 in_serverport, CPartFile* in_reqfile
#ifdef OLD_SLS_ENABLED
, CSourceEntry* source_entry = (CSourceEntry*)NULL
#endif //OLD_SLS_ENABLED
	);
	~CUpDownClient();
	void			Disconnected();
	void			TryToConnect(bool bIgnoreMaxCon = false);
	void			ConnectionEstablished();
	uint32			GetUserID()					{return m_nUserID;}
	char*			GetUserName()				{return m_pszUsername;}
	uint32			GetIP()						{return m_dwUserIP;}
	bool			HasLowID()					{return m_bIsLowID;}
	char*			GetFullIP()					{return m_szFullUserIP;}
	uint32			GetUserPort()				{return m_nUserPort;}
	uint32			GetTransferedUp()			{return m_nTransferedUp;}
	uint32			GetTransferedDown()			{return m_nTransferedDown;}
	uint32			GetServerIP()				{return m_dwServerIP;}
	uint16			GetServerPort()				{return m_nServerPort;}
	uchar*			GetUserHash()				{return (uchar*)m_achUserHash;}
	bool			HasValidHash()				{return ((int*)m_achUserHash)[0] != 0 || ((int*)m_achUserHash[1]) != 0 ||
												        ((int*)m_achUserHash)[2] != 0 || ((int*)m_achUserHash[3]) != 0; }
	uint32			GetVersion()				{return m_nClientVersion;}
#ifdef OLD_SLS_ENABLED
	const CSourceEntry*	const GetSourceEntry() const		{return m_sourceEntry;} // Added by Tarod to remove dupes
#endif //OLD_SLS_ENABLED
	uint8			GetMuleVersion()			{return m_byEmuleVersion;}
	char*			GetModVersion()			{ return (m_modVerString[0])? m_modVerString:NULL; }	//StaTiK - mods tag
	bool			ExtProtocolAvailable()		{return m_bEmuleProtocol;}
	bool			TarodVersion()				{return (m_byDataTarodVer == 1 || m_byDataTarodVer == 2);} // Added by Tarod
	uint32			PlusVersion()				{return m_dwDataPlusVers;}
	bool			IsEmuleClient()				{return m_byEmuleVersion;}
	CClientCredits* Credits()					{return credits;}
	bool			IsBanned() const			{return (m_bBanned && m_nDownloadState != DS_DOWNLOADING);}
	char*			GetClientFilename()			{return m_pszClientFilename;}
	bool			SupportsUDP()				{return m_byUDPVer != 0 && m_nUDPPort != 0;}
	uint16			GetUDPPort()				{return m_nUDPPort;}
	void			SetUDPPort(uint16 nPort)	{ m_nUDPPort = nPort; }
	uint8			GetUDPVersion()				{return m_byUDPVer;}
	uint8			GetExtendedRequestsVersion(){return m_byExtendedRequestsVer;}
	bool			IsFriend()	const			{return m_Friend != NULL;}
	float			GetCompression()	{return (float)compressiongain/notcompressed*100.0f;} // Add rod show compression
	void			ResetCompressionGain() {compressiongain = 0; notcompressed=1;} // Add show compression

	void			RequestSharedFileList();
	void			ProcessSharedFileList(char* pachPacket, uint32 nSize);
	
	void			SetUserName(char* pszNewName);
	bool            IsCommunity();  	// i_a                
	void            ResetValueCache(); 	// i_a                
	eClientTypes	GetClientSoft()				{return m_clientSoft;}
	void			ReGetClientSoft();
	void			ProcessHelloAnswer(char* pachPacket, uint32 nSize);
	void			ProcessHelloPacket(char* pachPacket, uint32 nSize);
	void			SendHelloAnswer();
	void			SendHelloPacket();
	void			SendMuleInfoPacket(bool bAnswer);
	void			ProcessMuleInfoPacket(char* pachPacket, uint32 nSize);
	void			ProcessMuleCommentPacket(char* pachPacket, uint32 nSize);
	bool			Compare(CUpDownClient* tocomp);
	void			SetLastSrcReqTime()			{m_dwLastSourceRequest = ::GetTickCount();}
	void			SetLastSrcAnswerTime()		{m_dwLastSourceAnswer = ::GetTickCount();}
	uint32			GetLastSrcReqTime()			{return m_dwLastSourceRequest;}
	uint32			GetLastSrcAnswerTime()		{return m_dwLastSourceAnswer;}
	bool			GetFriendSlot()				{return m_bFriendSlot;}
	void			SetFriendSlot(bool bNV)		{m_bFriendSlot = bNV;}
	void			SetCommentDirty(bool bDirty = true) {m_bCommentDirty = bDirty;}
	uint8			GetSourceExchangeVersion()	{return m_bySourceExchangeVer;}

	CClientReqSocket*	socket;
	CClientCredits*		credits;
	CFriend*			m_Friend;
	//upload
	uint32	compressiongain; /// Add show compression
	uint32  notcompressed; // Add show compression
	eULQState		GetUploadState()			{return m_byUploadState;}
	void			SetUploadState(eULQState news)	{m_byUploadState = news;}
	uint32			GetWaitStartTime()			{return m_dwWaitTime;}
	uint32			GetWaitTime()				{return m_dwUploadTime-m_dwWaitTime;}
	bool			IsDownloading()				{return (m_byUploadState == US_UPLOADING);}
	bool			HasBlocks()					{return !(m_BlockSend_queue.IsEmpty() && m_BlockRequests_queue.IsEmpty());}
	sint32			GetDatarate()				{return m_nUpDatarate;}	
	uint32			GetScore(bool sysvalue, bool isdownloading = false, bool onlybasevalue = false);
	float			GetRareFilePushRatio(void) ; // Added by Tarod
	float			GetSmallFilePushRatio(void) ; // Added by Tarod
	void			AddReqBlock(Requested_Block_Struct* reqblock);
	bool			CreateNextBlockPackage();
	void			SetUpStartTime(uint32 dwTime = 0);
	uint32			GetUpStartTimeDelay()		{return ::GetTickCount() - m_dwUploadTime;}
	void			SetWaitStartTime(uint32 dwTime = 0);
	void			SendHashsetPacket(char* forfileid);
	uchar			reqfileid[16];
	void			SetUploadFileID(uchar* tempreqfileid);
	uchar*			GetUploadFileID()	{return reqfileid;}
	uint32			SendBlockData(uint32 maxammount, bool& wantsMoreBandwidth);
	void			ClearUploadBlockRequests();
	void			SendRankingInfo();
	void			SendCommentInfo(CKnownFile *file);
	uint32			GetLastAskedDelay();
	void			AddRequestCount(uchar* fileid);
	bool 			IsDifferentPartBlock(bool silent=false);
	void			UnBan();
	void			Ban();
	uint32			GetBanTime()				{return m_dwBanTime;}
	uint32			GetAskedCount()				{return m_cAsked;} //<<--
	void			AddAskedCount()				{m_cAsked++;}
	void			SetAskedCount( uint32 m_cInAsked)	{m_cAsked = m_cInAsked;}
	void			FlushSendBlocks();			// call this when you stop upload, or the socket might be not able to send
	void			SetLastUpRequest()			{m_dwLastUpRequest = ::GetTickCount();}
	uint32			GetLastUpRequest()			{return m_dwLastUpRequest;}
	void			UDPFileReasked();

	uint32			GetSessionUp()			{return m_nTransferedUp - m_nCurSessionUp;}
	void			ResetSessionUp()		{m_nCurSessionUp = m_nTransferedUp;} 
    void            SetLastGotULData()         { m_LastGotULData = ::GetTickCount(); };
    DWORD           GetLastGotULData()         { return m_LastGotULData; };
    uint32			GetCurrentlyUploadingPart();

	// remote client's parts
	void			ProcessUpFileStatus(char* packet,uint32 size);
	uint16			GetUpPartCount()			{return m_nUpPartCount;}
	uint16			GetAvailUpPartCount()			{return m_nAvailUpPartCount;}
	void			DrawUpStatusBar(CDC* dc, RECT* rect, bool onlygreyrect, bool  bFlat);
    void            CheckForUpPartsInfo();

	//download
	uint32			GetAskedCountDown()				{return m_cDownAsked;} //<<--
	void			AddAskedCountDown()				{m_cDownAsked++;}
	void			SetAskedCountDown( uint32 m_cInDownAsked)				{m_cDownAsked = m_cInDownAsked;}
	uint8			GetDownloadState()			{return m_nDownloadState;}
	void			SetDownloadState(uint8 byNewState);
	uint32			GetLastAskedTime()			{return m_dwLastAskedTime;}
	bool			IsPartAvailable(uint16 iPart)	{return	( (iPart >= m_nPartCount) || (!m_abyPartStatus) )? 0:m_abyPartStatus[iPart];} 	
	uint8*			GetPartStatus()				{return m_abyPartStatus;}
	uint32			GetDownloadDatarate()		{return m_nDownDatarate;}
	uint16			GetRemoteQueueRank()		{return m_nRemoteQueueRank;}
	void			SetRemoteQueueFull( bool flag )	{m_bRemoteQueueFull = flag;}
	bool			IsRemoteQueueFull()			{return m_bRemoteQueueFull;}
	void			SetRemoteQueueRank(uint16 nr);
	void			DrawStatusBar(CDC* dc, RECT* rect, bool onlygreyrect, bool  bFlat);
	void			AskForDownload();
	void			SendFileRequest();
//	void			SendFileRequestForUpload();
	void			ProcessFileInfo(char* packet,uint32 size);
	void			ProcessFileStatus(char* packet,uint32 size);
	void			ProcessHashSet(char* packet,uint32 size);
	bool			AddRequestForAnotherFile(CPartFile* file);
	void			SendBlockRequests();
	void			ProcessBlockPacket(char* packet, uint32 size, bool packed = false);
	sint32		    CalculateDownloadRate();
	uint16			GetAvailablePartCount();
	bool			SwapToAnotherFile(bool bIgnoreNoNeeded = false, bool bManualChange = false);
	void			HandoverToAnotherFile(CPartFile* file);      // A4AF - Amdribant 
    uint32			GetQueueWaitTime()		{return ::GetTickCount() - m_dwQueueWaitTime;} //<<-- enkeyDEV(Ottavio84) -Show download queue wait time-
	void			StartQueueWaitTimer()	{m_dwQueueWaitTime = ::GetTickCount();}	 //<<-- enkeyDEV(Ottavio84) -Show download queue wait time-
	void			UDPReaskACK(uint16 nNewQR);
	void			UDPReaskFNF();
	void			UDPReaskForDownload();
	bool			IsSourceRequestAllowed();
	uint32			GetRemoteScore(); //Maverick
	uint16			GetRemoteRatio(); //Maverick
	float			GetRemoteBaseModifier(); //Maverick
	bool			GetRemoteCommunity(){return (m_RemoteInfoAvaiable && m_RemoteCommunity);} //Maverick
	bool			GetRemoteCommunityEnabled(){return (m_RemoteInfoAvaiable && m_RemoteCommunityEnabled);} //Maverick
	void			SetRemoteScore(uint32 Score){m_RemoteScore = Score; m_RemoteInfoAvaiable=true;} //Maverick
	void			SetRemoteRatio(uint16 Ratio){m_RemoteRatio = Ratio; m_RemoteInfoAvaiable=true;} // Maverick
	void			SetRemoteBaseModifier(uint8 BaseModifier){m_RemoteBaseModifier = BaseModifier * 10; m_RemoteInfoAvaiable=true;} // Maverick
	void			SetRemoteCommunity(bool Community){m_RemoteCommunity = Community; m_RemoteInfoAvaiable=true;} //Maverick
	void			SetRemoteCommunityEnabled(bool CommunityEnabled){m_RemoteCommunityEnabled = CommunityEnabled; m_RemoteInfoAvaiable=true;} //Maverick
	bool			IsRemoteInfoAvaiable(){return m_RemoteInfoAvaiable;} //Maverick
	void			SetRemoteClientSpeed(uint16 speed){m_RemoteClientSpeed = speed;} //Maverick
	uint16			GetRemoteClientSpeed(){return m_RemoteClientSpeed;} // Maverick

	CString			GetDownloadStateAsString(void); // Lord KiRon

	CPartFile*		reqfile;
	int				sourcesslot;
		
	//returns the count of avaible parts that this user has but we not have
	uint16	GetAvailableOfNeededPartsCount(); //added by Mr.Fry

	//chat
	uint8			GetChatState()				{return m_byChatstate;}
	void			SetChatState(uint8 nNewS)	{m_byChatstate = nNewS;}

	//File Comment 
	CString         GetFileComment()         	{return m_strComment;} 
	void		   	SetFileComment(char *desc) 	{m_strComment.Format("%s",desc);}
	uint8       	GetFileRate()            	{return m_iRate;}
	void		   	SetFileRate(int8 iNewRate) 	{m_iRate=iNewRate;}

	// Barry - Process zip file as it arrives, don't need to wait until end of block
	int unzip(Pending_Block_Struct *block, BYTE *zipped, uint32 lenZipped, BYTE **unzipped, uint32 *lenUnzipped, bool recursive = false);
	void ShowDownloadingParts(_Bvector& parts);
	//START enkeyDEV(kei-kun) -Messaging preferences-	
	uint32			GetAwayMessageResendCount()				{return m_awayMessageResendCount;};
	void			SetAwayMessageResendCount(uint32 newVal) {m_awayMessageResendCount = newVal;};
	//END enkeyDEV(kei-kun) -Messaging preferences-
	void UpdateDisplayedInfo(boolean force=false);

	// LANCAST (moosetea) returns true if this client is is on the LAN and LANCAST is enabled!
	bool isOnLAN();
	uint16 m_LastPartAsked; //jicxicmic

	void CleanOtherRequests()	{m_OtherRequests_list.RemoveAll();m_OtherNoNeeded_list.RemoveAll();}	//Cax2 - remove sources bugfix
private:
	// base
	void	Init();
    void    SetUserID(uint32 newUserID) {
        m_nUserID = newUserID;
        m_bIsLowID = (m_nUserID < 16777216);
    }
	void	ProcessHelloTypePacket(CSafeMemFile* data);
	void	SendHelloTypePacket(CMemFile* data);
	float	GetClientSpeed();		// Maverick
	//bool		m_bIsBotuser;		//Cax2: removed punishment 
	bool	isfriend;				// Tarod (herbert code)
	bool    isemuleclient;   		// Tarod (herbert code) 
	bool    iscommunity;    		// DonGato 
    bool    community_cached;    	// i_a: CPU load reduction 
    bool    score_base_cached;      // i_a: CPU load reduction 
	uint32	m_dwUserIP;
	uint32	m_dwServerIP;
	uint32	m_nUserID;
    bool    m_bIsLowID;
	int16	m_nUserPort;
	int16	m_nServerPort;
	uint32	m_nClientVersion;
	sint32	m_nUpDatarate;
	//uint32	m_cSendblock;
	uint8	m_byEmuleVersion;
	uint8	m_byDataCompVer;
	uint8	m_byDataTarodVer; // Added by Tarod (just curiosity)
	uint32   m_dwDataPlusVers;
	char m_modVerString[40];		//Cax2 - mods tag
	bool	m_bEmuleProtocol;
	char*	m_pszUsername;
	char	m_szFullUserIP[21];
	char	m_achUserHash[16];
	uint16	m_nUDPPort;
	uint8	m_byUDPVer;
	uint8	m_bySourceExchangeVer;
	uint8	m_byAcceptCommentVer;
	uint8	m_byExtendedRequestsVer;
	uint8	m_cFailed;
	eClientTypes	m_clientSoft;
	uint32	m_dwLastSourceRequest;
	uint32	m_dwLastSourceAnswer;
	bool	m_bFileListRequested;
	bool	m_bFriendSlot;
	bool	m_bCommentDirty;
	bool	m_bIsHybrid;
	uint32	m_byCompatableClient;
	uint32	m_RemoteScore; //By Maverick
	uint16	m_RemoteRatio; //By Maverick
	uint16	m_RemoteBaseModifier; //By Maverick
	bool	m_RemoteInfoAvaiable; //By Maverick
	bool	m_RemoteCommunity; //By Maverick
	bool	m_RemoteCommunityEnabled; // By Maverick
	uint16	m_RemoteClientSpeed;
	CTypedPtrList<CPtrList, Packet*>		 m_WaitingPackets_list;
	DWORD	m_lastRefreshedDLDisplay;
#ifdef OLD_SLS_ENABLED
	CSourceEntry*	m_sourceEntry;	// Added by Tarod (LS tweak)
#endif //OLD_SLS_ENABLED

	//upload
	void CreateStandartPackets(byte* data,uint32 togo, Requested_Block_Struct* currentblock);
	void CreatePackedPackets(byte* data,uint32 togo, Requested_Block_Struct* currentblock);
    void CUpDownClient::splitAndAddPacketsToSendQueue(Packet* packet);
	bool		m_bBanned;
	uint32		m_nTransferedUp;
	eULQState	m_byUploadState;
	uint32		m_dwWaitTime;
	uint32		m_dwUploadTime;
	uint32		m_nMaxSendAllowed;
	uint32		m_nAvUpDatarate;
	uint32		m_cAsked; //<<--
	uint32		m_dwLastUpRequest;
	uint32		m_dwBanTime;
	int16		filepriority;	// Added by Tarod (herbert code)
	bool		m_bUsedComprUp;	//only used for interface output
	uint32		m_nCurSessionUp; // VQB add for UL slot control
	CList<int,int>									 m_AvarageUDR_list;
	CList<DWORD,DWORD>                               m_avarage_UL_tick_list;
	CTypedPtrList<CPtrList, Packet*>		 			m_BlockSend_queue;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> 	m_BlockRequests_queue;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> 	m_DoneBlocks_list;
	CTypedPtrList<CPtrList, Requested_File_Struct*>	 	m_RequestedFiles_list;
    DWORD       m_lastRefreshedULDisplay;
    uint32 m_LastGotULData;

	// remote client's parts
	uint16		m_nUpPartCount;
	uint16		m_nAvailUpPartCount;
	static		CBarShader s_UpStatusBar;
	public:
	uint8*		m_abyUpPartStatus;
	private:

	//download
	bool		m_bRemoteQueueFull;
	bool		usedcompressiondown; //only used for interface output
	uint8		m_nDownloadState;
	uint16		m_nPartCount;
	uint32		m_cDownAsked;
	uint8*		m_abyPartStatus;
	uint32		m_dwLastAskedTime;
	char*		m_pszClientFilename;
	uint32		m_nTransferedDown;
	uint32      m_nLastBlockOffset;   //<--NEW Cax2 Packet ? from
	uint32		m_nDownDatarate;
	uint32		m_nAvDownDatarate;
	//uint16		m_cShowDR;
	uint32		m_dwLastBlockReceived;
	uint16		m_nRemoteQueueRank;
	bool		m_bCompleteSource;
	bool		m_bReaskPending;
	bool		m_bUDPPending;
    uint32		m_dwQueueWaitTime; //<<-- enkeyDEV(Ottavio84) -Show download queue wait time-
//	CList<TransferredData,TransferredData>				m_AvarageDDR_list;
	CList<int,int>									 m_AvarageDDR_list;
	CList<DWORD,DWORD>                               m_avarage_DL_tick_list;
	CTypedPtrList<CPtrList, Pending_Block_Struct*>	 	m_PendingBlocks_list;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> 	m_DownloadBlocks_list;
	CTypedPtrList<CPtrList, CPartFile*>		 			m_OtherRequests_list;
	CTypedPtrList<CPtrList, CPartFile*>		 			m_OtherNoNeeded_list;

    DWORD       m_lastCalculatedDownloadRate;

//	uint32	sumavgddr; // enkeyDEV(Ottavio84) -Accurate speed measurement-
//	uint32	sumavgudr; // enkeyDEV(Ottavio84) -Accurate speed measurement-

	static CBarShader s_StatusBar;
	// chat
	uint8 m_byChatstate;
	CString m_strComment;
	int8 m_iRate; 
	uint32 m_awayMessageResendCount; //<<-- enkeyDEV(kei-kun) -Messaging preferences-
};
